//
//  ========================================================================
//  Copyright (c) 1995-2017 Mort Bay Consulting Pty. Ltd.
//  ------------------------------------------------------------------------
//  All rights reserved. This program and the accompanying materials
//  are made available under the terms of the Eclipse Public License v1.0
//  and Apache License v2.0 which accompanies this distribution.
//
//      The Eclipse Public License is available at
//      http://www.eclipse.org/legal/epl-v10.html
//
//      The Apache License v2.0 is available at
//      http://www.opensource.org/licenses/apache2.0.php
//
//  You may elect to redistribute this code under either of these licenses.
//  ========================================================================
//

package org.eclipse.jetty.websocket.jsr356.decoders;

import javax.websocket.DecodeException;
import javax.websocket.Decoder;

/**
 * Default implementation of the Text Message to {@link Float} decoder
 */
public class FloatDecoder extends AbstractDecoder implements Decoder.Text<Float>
{
    public static final FloatDecoder INSTANCE = new FloatDecoder();

    @Override
    public Float decode(String s) throws DecodeException
    {
        try
        {
            Float val = Float.parseFloat(s);
            if (val.isNaN())
            {
                throw new DecodeException(s,"NaN");
            }
            return val;
        }
        catch (NumberFormatException e)
        {
            throw new DecodeException(s,"Unable to parse float",e);
        }
    }

    @Override
    public boolean willDecode(String s)
    {
        if (s == null)
        {
            return false;
        }
        try
        {
            Float val = Float.parseFloat(s);
            return (!val.isNaN());
        }
        catch (NumberFormatException e)
        {
            return false;
        }
    }
}
